// main.cpp
#include "global.h"

GLOBALDATA gl_Data;

// The main program entry point. Differs for Windows and Linux.
#if PLATFORM == MY_PLATFORM_WIN32
// ------------------------------------------------------------------------------------------------
irr::s32 __stdcall WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, irr::s32 nCmdShow)
#else
irr::s32 main(irr::s32 argc, char **argv)
// ------------------------------------------------------------------------------------------------
#endif
{
	if (!gl_Init()) return(0); // Abort if initialization fails
	if (!gl_InitScreen()) return(0);	
	if (!gl_InitObjects()) return(0);
	
	// -------------------------------------------------
	while(gl_Data.pIrrlichtDevice->run())
	{
		irr::u32 fOldtime = gl_Data.pIrrlichtDevice->getTimer()->getRealTime(); 
		
		gl_Data.pVideoDriver->beginScene(true, true, irr::video::SColor(0,0,0,0)); // Clear backbuffer, clear zbuffer, color
		gl_Data.pSceneManager->drawAll(); // Draw all objects previously stored to the engine
		gl_Data.pGUIEnv->drawAll();
		gl_Data.pVideoDriver->endScene();

		gl_Loop();
	}
	// -------------------------------------------------

	gl_Exit();	

	return(0);
}

// ------------------------------------------------------------------------------------------------
bool gl_Init(void)
// ------------------------------------------------------------------------------------------------
{
	srand( (unsigned)time(NULL));
	
	// -------------------------------------------------
	gl_Data.ScreenXY.Width  = 1024; 
	gl_Data.ScreenXY.Height = 768;
	gl_Data.uColorDepth = 32; // ignored in windowed mode
	
	#if PLATFORM == MY_PLATFORM_WIN32 // -------------------------------------- WIN32

		gl_Data.pIrrlichtDevice = irr::createDevice(
								irr::video::EDT_DIRECT3D9, // EDT_NULL, EDT_SOFTWARE, EDT_DIRECT3D8, EDT_DIRECT3D9, EDT_OPENGL
								gl_Data.ScreenXY, // Screen resolution
								gl_Data.uColorDepth, // bits per pixel (fullscreen only)
								false,  // fullscreen? better be false when debugging
								false,  // create stencil buffer (for e.g. stencil buffer shadows)
								true,   // use vsync
								&gl_Data.Input,   // an IEventReceiver* (user created). 
								IRRLICHT_SDK_VERSION);		

	#else  // -------------------------------------- LINUX
		
		gl_Data.pIrrlichtDevice = irr::createDevice(
								irr::video::EDT_OPENGL, // EDT_NULL, EDT_SOFTWARE, EDT_DIRECTX8, EDT_DIRECTX9, EDT_OPENGL. 
								gl_Data.ScreenXY, // Screen resolution. 
								gl_Data.uColorDepth, // bits per pixel (fullscreen only)
								false,    // fullscreen? better be false when debugging
								false,    // create stencil buffer (for e.g. stencil buffer shadows)
								false,    // use vsync
								&gl_Data.Input,	  // an IEventReceiver* (user created)
								IRRLICHT_SDK_VERSION);
		
	#endif // --------------------------------------
	
	if (!gl_Data.pIrrlichtDevice) return(false);

	gl_Data.pVideoDriver  = gl_Data.pIrrlichtDevice->getVideoDriver();
	if (!gl_Data.pVideoDriver) return(false);

	gl_Data.pSceneManager = gl_Data.pIrrlichtDevice->getSceneManager();
	if (!gl_Data.pSceneManager) return(false);

	gl_Data.pLogger       = gl_Data.pIrrlichtDevice->getLogger();
	if (!gl_Data.pLogger) return(false);

	// -------------------------------------------------
	// Driver presets

	gl_Data.pVideoDriver->setTextureCreationFlag(irr::video::ETCF_ALWAYS_32_BIT, true);

	return(true); // all seems ok
}

// ------------------------------------------------------------------------------------------------
bool gl_InitObjects(void)
// ------------------------------------------------------------------------------------------------
{
	// -------------------------------------------------
	// Objects initialisation

	if (!gl_Data.Camera.Init()) return(false);
	if (!gl_Data.Map.Init())	return(false);
	if (!gl_Data.Gui.Init())	return(false);
	if (!gl_Data.Input.Init())	return(false);
	if (!gl_Data.Debug.Init())	return(false);
	if (!gl_Data.Dice.Init())	return(false);
	
	// -------------------------------------------------

	for (irr::s32 i=0;i<4; i++) gl_Data.Player[i].Init(i);
	gl_SetGameState(idle);

	return(true);
}

// ------------------------------------------------------------------------------------------------
bool gl_InitScreen(void)
// ------------------------------------------------------------------------------------------------
{
	irr::video::ITexture* pT = gl_Data.pVideoDriver->getTexture("./data/CM_InitScreen.bmp");
	if (!pT) return(false);
	
	gl_Data.pVideoDriver->beginScene(true, true, irr::video::SColor(0,0,0,0)); // Clear backbuffer, clear zbuffer, color
	gl_Data.pVideoDriver->draw2DImage(pT,irr::core::position2d<irr::s32>(0,0),irr::core::rect<irr::s32>(0,0,1024,768));    
	gl_Data.pVideoDriver->endScene();

	return(true);
}

// ------------------------------------------------------------------------------------------------
void gl_Reset(void)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4; i++) gl_Data.Player[i].Reset();

	gl_Data.Camera.Reset();
	gl_Data.Map.Reset();
	gl_Data.Gui.Reset();
	gl_Data.Input.Reset();
	gl_Data.Debug.Reset();
	gl_Data.Dice.Reset();

	gl_Data.bGamePaused = false;
	gl_Data.iActivePlayer = -1;
}	
// ------------------------------------------------------------------------------------------------
void gl_Exit(void)
// ------------------------------------------------------------------------------------------------
{
	for (irr::s32 i=0;i<4; i++) gl_Data.Player[i].Exit();
	
	gl_Data.Camera.Exit();
	gl_Data.Map.Exit();
	gl_Data.Gui.Exit();
	gl_Data.Input.Exit();
	gl_Data.Debug.Exit();
	gl_Data.Dice.Exit();
	
	gl_Data.pIrrlichtDevice->drop();
}

// ------------------------------------------------------------------------------------------------
void gl_Loop(void)
// ------------------------------------------------------------------------------------------------
{
	gl_Data.Camera.Loop();
	gl_Data.Map.Loop();
	gl_Data.Gui.Loop();
	gl_Data.Input.Loop();
	gl_Data.Debug.Loop();
	gl_Data.Dice.Loop();

	if (gl_Data.bGamePaused) return;
	
	if ((gl_Data.GameState==autogame))
	{
		bool bIsFinished = gl_Data.Player[gl_Data.iActivePlayer].Loop();
		if (bIsFinished)
		{
			if (gl_Data.Map.CheckIfPlayerHasWon(gl_Data.iActivePlayer))
			{
				gl_Data.bGamePaused = true;
				gl_Data.Gui.SetPauseSign(true);
				gl_Data.Gui.ShowWinner(gl_Data.iActivePlayer, true);
			}
						
			gl_Data.iActivePlayer++;
			if (gl_Data.iActivePlayer > 3) gl_Data.iActivePlayer = 0;
		}
	}
}

// ------------------------------------------------------------------------------------------------
void gl_SetGameState(E_GAME_STATE newState)
// ------------------------------------------------------------------------------------------------
{
	switch(newState)
	{
	case idle:    gl_Reset();
				  gl_Data.GameState = newState;
				  break;

	case autogame: gl_Reset();
				   gl_Data.GameState = newState;
				   gl_Data.Gui.SetRunningGame(true);
				   gl_Data.iActivePlayer = 0;    // Player 1 begins
				   break;
	}
}
